@@ -1,48 +1,11 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtQml module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qqmlbinding_p.h"
 
-#include "qqml.h"
 #include "qqmlcontext.h"
-#include "qqmlinfo.h"
 #include "qqmldata_p.h"
+#include "qqmlinfo.h"
 
 #include <private/qqmldebugserviceinterfaces_p.h>
 #include <private/qqmldebugconnector_p.h>
@@ -67,9 +30,12 @@
 
 QT_BEGIN_NAMESPACE
 
+Q_TRACE_POINT(qtqml, QQmlBinding_entry, const QQmlEngine *engine, const QString &function, const QString &fileName, int line, int column)
+Q_TRACE_POINT(qtqml, QQmlBinding_exit)
+
 QQmlBinding *QQmlBinding::create(const QQmlPropertyData *property, const QQmlScriptString &script, QObject *obj, QQmlContext *ctxt)
 {
-    QQmlBinding *b = newBinding(QQmlEnginePrivate::get(ctxt), property);
+    QQmlBinding *b = newBinding(property);
 
     if (ctxt && !ctxt->isValid())
         return b;
@@ -125,7 +91,7 @@ QQmlBinding *QQmlBinding::create(
         const QQmlPropertyData *property, const QString &str, QObject *obj,
         const QQmlRefPointer<QQmlContextData> &ctxt, const QString &url, quint16 lineNumber)
 {
-    QQmlBinding *b = newBinding(QQmlEnginePrivate::get(ctxt), property);
+    QQmlBinding *b = newBinding(property);
 
     b->setNotifyOnValueChanged(true);
     b->QQmlJavaScriptExpression::setContext(ctxt);
@@ -147,7 +113,7 @@ QQmlBinding *QQmlBinding::create(QMetaType propertyType, QV4::Function *function
                                  const QQmlRefPointer<QQmlContextData> &ctxt,
                                  QV4::ExecutionContext *scope)
 {
-    QQmlBinding *b = newBinding(QQmlEnginePrivate::get(ctxt), propertyType);
+    QQmlBinding *b = newBinding(propertyType);
 
     b->setNotifyOnValueChanged(true);
     b->QQmlJavaScriptExpression::setContext(ctxt);
@@ -175,12 +141,12 @@ void QQmlBinding::update(QQmlPropertyData::WriteFlags flags)
 
     // Check for a binding update loop
     if (Q_UNLIKELY(updatingFlag())) {
-        QQmlPropertyData *d = nullptr;
+        const QQmlPropertyData *d = nullptr;
         QQmlPropertyData vtd;
         getPropertyData(&d, &vtd);
         Q_ASSERT(d);
         QQmlProperty p = QQmlPropertyPrivate::restore(targetObject(), *d, &vtd, nullptr);
-        QQmlAbstractBinding::printBindingLoopError(p);
+        printBindingLoopError(p);
         return;
     }
     setUpdatingFlag(true);
@@ -202,6 +168,12 @@ void QQmlBinding::update(QQmlPropertyData::WriteFlags flags)
         setUpdatingFlag(false);
 }
 
+void QQmlBinding::printBindingLoopError(const QQmlProperty &prop)
+{
+    qmlWarning(prop.object()) << QString(QLatin1String("Binding loop detected for property \"%1\":\n%2"))
+        .arg(prop.name(), expressionIdentifier());
+}
+
 QV4::ReturnedValue QQmlBinding::evaluate(bool *isUndefined)
 {
     QV4::ExecutionEngine *v4 = engine()->handle();
@@ -223,98 +195,15 @@ QV4::ReturnedValue QQmlBinding::evaluate(bool *isUndefined)
     return QQmlJavaScriptExpression::evaluate(jsCall.callData(scope), isUndefined);
 }
 
-
-// QQmlBindingBinding is for target properties which are of type "binding" (instead of, say, int or
-// double). The reason for being is that GenericBinding::fastWrite needs a compile-time constant
-// expression for the switch for the compiler to generate the optimal code, but
-// qMetaTypeId<QQmlBinding *>() needs to be used for the ID. So QQmlBinding::newBinding uses that
-// to instantiate this class.
-class QQmlBindingBinding: public QQmlBinding
-{
-protected:
-    void doUpdate(const DeleteWatcher &,
-                  QQmlPropertyData::WriteFlags flags, QV4::Scope &) override final
-    {
-        Q_ASSERT(!m_targetIndex.hasValueTypeIndex());
-        QQmlPropertyData *pd = nullptr;
-        getPropertyData(&pd, nullptr);
-        QQmlBinding *thisPtr = this;
-        pd->writeProperty(m_target.data(), &thisPtr, flags);
-    }
-};
-
-// For any target that's not a binding, we have a common doUpdate. However, depending on the type
-// of the target property, there is a specialized write method.
-class QQmlNonbindingBinding: public QQmlBinding
-{
-protected:
-    void doUpdate(const DeleteWatcher &watcher,
-                  QQmlPropertyData::WriteFlags flags, QV4::Scope &scope) override
-    {
-        auto ep = QQmlEnginePrivate::get(scope.engine);
-        ep->referenceScarceResources();
-
-        bool error = false;
-        auto canWrite = [&]() { return !watcher.wasDeleted() && isAddedToObject() && !hasError(); };
-        const QV4::Function *v4Function = function();
-        if (v4Function && v4Function->aotFunction && !hasBoundFunction()) {
-            const auto returnType = v4Function->aotFunction->returnType;
-            if (returnType == QMetaType::fromType<QVariant>()) {
-                // It expects uninitialized memory
-                Q_ALLOCA_VAR(QVariant, result, sizeof(QVariant));
-                const bool isUndefined = !evaluate(result, returnType);
-                if (canWrite())
-                    error = !write(result->data(), result->metaType(), isUndefined, flags);
-                result->~QVariant();
-            } else {
-                const auto size = returnType.sizeOf();
-                if (Q_LIKELY(size > 0)) {
-                    Q_ALLOCA_VAR(void, result, size);
-                    const bool isUndefined = !evaluate(result, returnType);
-                    if (canWrite())
-                        error = !write(result, returnType, isUndefined, flags);
-                    returnType.destruct(result);
-                } else if (canWrite()) {
-                    error = !write(QV4::Encode::undefined(), true, flags);
-                }
-            }
-        } else {
-            bool isUndefined = false;
-            QV4::ScopedValue result(scope, evaluate(&isUndefined));
-            if (canWrite())
-                error = !write(result, isUndefined, flags);
-        }
-
-        if (!watcher.wasDeleted()) {
-
-            if (error) {
-                delayedError()->setErrorLocation(sourceLocation());
-                delayedError()->setErrorObject(m_target.data());
-            }
-
-            if (hasError()) {
-                if (!delayedError()->addError(ep)) ep->warning(this->error(engine()));
-            } else {
-                clearError();
-            }
-        }
-
-        ep->dereferenceScarceResources();
-    }
-
-    virtual bool write(const QV4::Value &result, bool isUndefined, QQmlPropertyData::WriteFlags flags) = 0;
-    virtual bool write(void *result, QMetaType type, bool isUndefined, QQmlPropertyData::WriteFlags flags) = 0;
-};
-
 template<int StaticPropType>
-class GenericBinding: public QQmlNonbindingBinding
+class GenericBinding: public QQmlBinding
 {
 protected:
     // Returns true if successful, false if an error description was set on expression
     Q_ALWAYS_INLINE bool write(void *result, QMetaType type, bool isUndefined,
                                QQmlPropertyData::WriteFlags flags) override final
     {
-        QQmlPropertyData *pd;
+        const QQmlPropertyData *pd;
         QQmlPropertyData vpd;
         getPropertyData(&pd, &vpd);
         Q_ASSERT(pd);
@@ -338,7 +227,7 @@ protected:
     {
         Q_ASSERT(targetObject());
 
-        QQmlPropertyData *pd;
+        const QQmlPropertyData *pd;
         QQmlPropertyData vpd;
         getPropertyData(&pd, &vpd);
         Q_ASSERT(pd);
@@ -375,7 +264,7 @@ protected:
                 break;
             default:
                 if (const QV4::QQmlValueTypeWrapper *vtw = result.as<const QV4::QQmlValueTypeWrapper>()) {
-                    if (vtw->d()->valueType()->metaType == pd->propType()) {
+                    if (vtw->d()->valueType()->metaType() == pd->propType()) {
                         return vtw->write(m_target.data(), pd->coreIndex());
                     }
                 }
@@ -396,20 +285,14 @@ protected:
 
 class QQmlTranslationBinding : public GenericBinding<QMetaType::QString>, public QPropertyObserver {
 public:
-    QQmlTranslationBinding(const QQmlRefPointer<QV4::ExecutableCompilationUnit> &compilationUnit, const QV4::CompiledData::Binding *binding)
+    QQmlTranslationBinding(const QQmlRefPointer<QV4::ExecutableCompilationUnit> &compilationUnit)
         : QPropertyObserver(&QQmlTranslationBinding::onLanguageChange)
     {
         setCompilationUnit(compilationUnit);
-        m_binding = binding;
         setSource(QQmlEnginePrivate::get(compilationUnit->engine)->translationLanguage);
     }
 
-    QQmlSourceLocation sourceLocation() const override final
-    {
-        return QQmlSourceLocation(
-                m_compilationUnit->fileName(), m_binding->valueLocation.line(),
-                m_binding->valueLocation.column());
-    }
+    virtual QString bindingValue() const = 0;
 
     static void onLanguageChange(QPropertyObserver *observer, QUntypedPropertyData *)
     { static_cast<QQmlTranslationBinding *>(observer)->update(); }
@@ -423,11 +306,11 @@ public:
         if (!isAddedToObject() || hasError())
             return;
 
-        const QString result = m_compilationUnit->bindingValueAsString(m_binding);
+        const QString result = this->bindingValue();
 
         Q_ASSERT(targetObject());
 
-        QQmlPropertyData *pd;
+        const QQmlPropertyData *pd;
         QQmlPropertyData vpd;
         getPropertyData(&pd, &vpd);
         Q_ASSERT(pd);
@@ -440,9 +323,56 @@ public:
     }
 
     bool hasDependencies() const override final { return true; }
+};
 
-private:
+class QQmlTranslationBindingFromBinding : public QQmlTranslationBinding
+{
     const QV4::CompiledData::Binding *m_binding;
+
+public:
+    QQmlTranslationBindingFromBinding(
+            const QQmlRefPointer<QV4::ExecutableCompilationUnit> &compilationUnit,
+            const QV4::CompiledData::Binding *binding)
+        : QQmlTranslationBinding(compilationUnit), m_binding(binding)
+    {
+    }
+
+    QString bindingValue() const override
+    {
+        return this->m_compilationUnit->bindingValueAsString(m_binding);
+    }
+
+    QQmlSourceLocation sourceLocation() const override final
+    {
+        return QQmlSourceLocation(m_compilationUnit->fileName(), m_binding->valueLocation.line(),
+                                  m_binding->valueLocation.column());
+    }
+};
+
+class QQmlTranslationBindingFromTranslationInfo : public QQmlTranslationBinding
+{
+    QQmlTranslation m_translationData;
+
+    quint16 m_line;
+    quint16 m_column;
+
+public:
+    QQmlTranslationBindingFromTranslationInfo(
+            const QQmlRefPointer<QV4::ExecutableCompilationUnit> &compilationUnit,
+            const QQmlTranslation &translationData, quint16 line, quint16 column)
+        : QQmlTranslationBinding(compilationUnit),
+          m_translationData(translationData),
+          m_line(line),
+          m_column(column)
+    {
+    }
+
+    virtual QString bindingValue() const override { return m_translationData.translate(); }
+
+    QQmlSourceLocation sourceLocation() const override final
+    {
+        return QQmlSourceLocation(m_compilationUnit->fileName(), m_line, m_column);
+    }
 };
 
 QQmlBinding *QQmlBinding::createTranslationBinding(
@@ -450,7 +380,7 @@ QQmlBinding *QQmlBinding::createTranslationBinding(
         const QV4::CompiledData::Binding *binding, QObject *obj,
         const QQmlRefPointer<QQmlContextData> &ctxt)
 {
-    QQmlTranslationBinding *b = new QQmlTranslationBinding(unit, binding);
+    QQmlTranslationBinding *b = new QQmlTranslationBindingFromBinding(unit, binding);
 
     b->setNotifyOnValueChanged(true);
     b->QQmlJavaScriptExpression::setContext(ctxt);
@@ -458,7 +388,34 @@ QQmlBinding *QQmlBinding::createTranslationBinding(
 #if QT_CONFIG(translation) && QT_CONFIG(qml_debug)
     if (QQmlDebugTranslationService *service
                  = QQmlDebugConnector::service<QQmlDebugTranslationService>()) {
-        service->foundTranslationBinding({unit, binding, b->scopeObject(), ctxt});
+        service->foundTranslationBinding(
+                TranslationBindingInformation::create(unit, binding, b->scopeObject(), ctxt));
+    }
+#endif
+    return b;
+}
+
+QQmlBinding *QQmlBinding::createTranslationBinding(
+        const QQmlRefPointer<QV4::ExecutableCompilationUnit> &unit,
+        const QQmlRefPointer<QQmlContextData> &ctxt, const QString &propertyName,
+        const QQmlTranslation &translationData, const QQmlSourceLocation &location, QObject *obj)
+{
+    QQmlTranslationBinding *b = new QQmlTranslationBindingFromTranslationInfo(
+            unit, translationData, location.column, location.line);
+
+    b->setNotifyOnValueChanged(true);
+    b->QQmlJavaScriptExpression::setContext(ctxt);
+    b->setScopeObject(obj);
+
+#if QT_CONFIG(translation) && QT_CONFIG(qml_debug)
+    QString originString;
+    if (QQmlDebugTranslationService *service =
+                QQmlDebugConnector::service<QQmlDebugTranslationService>()) {
+        service->foundTranslationBinding({ unit, b->scopeObject(), ctxt,
+
+                                           propertyName, translationData,
+
+                                           location.line, location.column });
     }
 #endif
     return b;
@@ -519,9 +476,6 @@ Q_NEVER_INLINE bool QQmlBinding::slowWrite(const QQmlPropertyData &core,
                                            const QV4::Value &result,
                                            bool isUndefined, QQmlPropertyData::WriteFlags flags)
 {
-    QQmlEngine *qmlEngine = engine();
-    QV4::ExecutionEngine *v4engine = qmlEngine->handle();
-
     const QMetaType metaType = valueTypeData.isValid() ? valueTypeData.propType() : core.propType();
     const int type = metaType.id();
 
@@ -532,17 +486,20 @@ Q_NEVER_INLINE bool QQmlBinding::slowWrite(const QQmlPropertyData &core,
 
     if (isUndefined) {
     } else if (core.isQList()) {
-        value = v4engine->toVariant(result, QMetaType::fromType<QList<QObject *> >());
+        if (core.propType().flags() & QMetaType::IsQmlList)
+            value = QV4::ExecutionEngine::toVariant(result, QMetaType::fromType<QList<QObject*>>());
+        else
+            value = QV4::ExecutionEngine::toVariant(result, core.propType());
     } else if (result.isNull() && core.isQObject()) {
         value = QVariant::fromValue((QObject *)nullptr);
     } else if (core.propType() == QMetaType::fromType<QList<QUrl>>()) {
         const QVariant resultVariant
-                = v4engine->toVariant(result, QMetaType::fromType<QList<QUrl>>());
+                = QV4::ExecutionEngine::toVariant(result, QMetaType::fromType<QList<QUrl>>());
         value = QVariant::fromValue(QQmlPropertyPrivate::resolveUrlsOnAssignment()
                                     ? QQmlPropertyPrivate::urlSequence(resultVariant, context())
                                     : QQmlPropertyPrivate::urlSequence(resultVariant));
     } else if (!isVarProperty && metaType != QMetaType::fromType<QJSValue>()) {
-        value = v4engine->toVariant(result, metaType);
+        value = QV4::ExecutionEngine::toVariant(result, metaType);
     }
 
     if (hasError()) {
@@ -559,9 +516,10 @@ Q_NEVER_INLINE bool QQmlBinding::slowWrite(const QQmlPropertyData &core,
         QQmlVMEMetaObject *vmemo = QQmlVMEMetaObject::get(m_target.data());
         Q_ASSERT(vmemo);
         vmemo->setVMEProperty(core.coreIndex(), result);
-    } else if (isUndefined && core.isResettable()) {
-        void *args[] = { nullptr };
-        QMetaObject::metacall(m_target.data(), QMetaObject::ResetProperty, core.coreIndex(), args);
+    } else if (isUndefined
+               && (valueTypeData.isValid() ? valueTypeData.isResettable() : core.isResettable())) {
+        QQmlPropertyPrivate::resetValueProperty(
+                    m_target.data(), core, valueTypeData, context(), flags);
     } else if (isUndefined && type == QMetaType::QVariant) {
         QQmlPropertyPrivate::writeValueProperty(m_target.data(), core, valueTypeData, QVariant(), context(), flags);
     } else if (metaType == QMetaType::fromType<QJSValue>()) {
@@ -605,8 +563,7 @@ void QQmlBinding::handleWriteError(const void *result, QMetaType resultType, QMe
     if (resultType.flags() & QMetaType::PointerToQObject) {
         if (QObject *o = *(QObject *const *)result) {
             valueType = o->metaObject()->className();
-            QQmlMetaObject propertyMetaObject = QQmlPropertyPrivate::rawMetaObjectForType(
-                        QQmlEnginePrivate::get(engine()), metaType.id());
+            QQmlMetaObject propertyMetaObject = QQmlPropertyPrivate::rawMetaObjectForType(metaType);
             if (!propertyMetaObject.isNull())
                 propertyType = propertyMetaObject.className();
         }
@@ -645,7 +602,7 @@ QVariant QQmlBinding::evaluate()
 
     ep->dereferenceScarceResources();
 
-    return scope.engine->toVariant(result, QMetaType::fromType<QList<QObject*> >());
+    return QV4::ExecutionEngine::toVariant(result, QMetaType::fromType<QList<QObject*> >());
 }
 
 void QQmlBinding::expressionChanged()
@@ -663,12 +620,7 @@ void QQmlBinding::setEnabled(bool e, QQmlPropertyData::WriteFlags flags)
     const bool wasEnabled = enabledFlag();
     setEnabledFlag(e);
     setNotifyOnValueChanged(e);
-
-    m_nextBinding.setTag(m_nextBinding.tag().setFlag(CanUseAccessor)); // Always use accessors, only not when:
-    if (auto interceptorMetaObject = QQmlInterceptorMetaObject::get(targetObject())) {
-        if (!m_targetIndex.isValid() || interceptorMetaObject->intercepts(m_targetIndex))
-            m_nextBinding.setTag(m_nextBinding.tag().setFlag(CanUseAccessor, false));
-    }
+    updateCanUseAccessor();
 
     if (e && !wasEnabled)
         update(flags);
@@ -679,113 +631,6 @@ QString QQmlBinding::expression() const
     return QStringLiteral("function() { [native code] }");
 }
 
-void QQmlBinding::setTarget(const QQmlProperty &prop)
-{
-    auto pd = QQmlPropertyPrivate::get(prop);
-    setTarget(prop.object(), pd->core, &pd->valueTypeData);
-}
-
-bool QQmlBinding::setTarget(QObject *object, const QQmlPropertyData &core, const QQmlPropertyData *valueType)
-{
-    return setTarget(object, core.coreIndex(), core.isAlias(),
-                     valueType ? valueType->coreIndex() : -1);
-}
-
-static const QQmlPropertyData *getObjectPropertyData(QObject *object, int coreIndex)
-{
-    QQmlData *data = QQmlData::get(object, true);
-    if (!data)
-        return nullptr;
-    if (!data->propertyCache) {
-        data->propertyCache = QQmlMetaType::propertyCache(object->metaObject());
-        if (!data->propertyCache)
-            return nullptr;
-        data->propertyCache->addref();
-    }
-    const QQmlPropertyData *propertyData = data->propertyCache->property(coreIndex);
-    Q_ASSERT(propertyData);
-    return propertyData;
-}
-
-bool QQmlBinding::setTarget(QObject *object, int coreIndex, bool coreIsAlias, int valueTypeIndex)
-{
-    auto invalidate = [this]() {
-        m_target = nullptr;
-        m_targetIndex = QQmlPropertyIndex();
-        return false;
-    };
-
-    if (!object)
-        return invalidate();
-
-    m_target = object;
-
-    for (bool isAlias = coreIsAlias; isAlias;) {
-        QQmlVMEMetaObject *vme = QQmlVMEMetaObject::getForProperty(object, coreIndex);
-
-        int aValueTypeIndex;
-        if (!vme->aliasTarget(coreIndex, &object, &coreIndex, &aValueTypeIndex)) {
-            // can't resolve id (yet)
-            return invalidate();
-        }
-
-        const QQmlPropertyData *propertyData = getObjectPropertyData(object, coreIndex);
-        if (!propertyData)
-            return invalidate();
-        if (aValueTypeIndex != -1) {
-            if (propertyData->propType().flags().testFlag(QMetaType::PointerToQObject)) {
-                // deep alias
-                propertyData->readProperty(object, &object);
-                coreIndex = aValueTypeIndex;
-                valueTypeIndex = -1;
-                propertyData = getObjectPropertyData(object, coreIndex);
-                if (!propertyData)
-                    return invalidate();
-            } else {
-                valueTypeIndex = aValueTypeIndex;
-            }
-        }
-
-        m_target = object;
-        isAlias = propertyData->isAlias();
-        coreIndex = propertyData->coreIndex();
-    }
-    m_targetIndex = QQmlPropertyIndex(coreIndex, valueTypeIndex);
-
-    QQmlData *data = QQmlData::get(m_target.data(), true);
-    if (!data->propertyCache) {
-        data->propertyCache = QQmlEnginePrivate::get(engine())->cache(m_target->metaObject());
-        data->propertyCache->addref();
-    }
-
-    return true;
-}
-
-void QQmlBinding::getPropertyData(QQmlPropertyData **propertyData, QQmlPropertyData *valueTypeData) const
-{
-    Q_ASSERT(propertyData);
-
-    QQmlData *data = QQmlData::get(m_target.data(), false);
-    Q_ASSERT(data);
-
-    if (Q_UNLIKELY(!data->propertyCache)) {
-        data->propertyCache = QQmlEnginePrivate::get(engine())->cache(m_target->metaObject());
-        data->propertyCache->addref();
-    }
-
-    *propertyData = data->propertyCache->property(m_targetIndex.coreIndex());
-    Q_ASSERT(*propertyData);
-
-    if (Q_UNLIKELY(m_targetIndex.hasValueTypeIndex() && valueTypeData)) {
-        const QMetaObject *valueTypeMetaObject = QQmlMetaType::metaObjectForValueType((*propertyData)->propType());
-        Q_ASSERT(valueTypeMetaObject);
-        QMetaProperty vtProp = valueTypeMetaObject->property(m_targetIndex.valueTypeIndex());
-        valueTypeData->setFlags(QQmlPropertyData::flagsForProperty(vtProp));
-        valueTypeData->setPropType(vtProp.metaType());
-        valueTypeData->setCoreIndex(m_targetIndex.valueTypeIndex());
-    }
-}
-
 QVector<QQmlProperty> QQmlBinding::dependencies() const
 {
     QVector<QQmlProperty> dependencies;
@@ -826,20 +671,73 @@ bool QQmlBinding::hasDependencies() const
     return !activeGuards.isEmpty() || qpropertyChangeTriggers;
 }
 
-class QObjectPointerBinding: public QQmlNonbindingBinding
+void QQmlBinding::doUpdate(const DeleteWatcher &watcher, QQmlPropertyData::WriteFlags flags, QV4::Scope &scope)
+{
+    auto ep = QQmlEnginePrivate::get(scope.engine);
+    ep->referenceScarceResources();
+
+    bool error = false;
+    auto canWrite = [&]() { return !watcher.wasDeleted() && isAddedToObject() && !hasError(); };
+    const QV4::Function *v4Function = function();
+    if (v4Function && v4Function->kind == QV4::Function::AotCompiled && !hasBoundFunction()) {
+        const auto returnType = v4Function->typedFunction->returnType;
+        if (returnType == QMetaType::fromType<QVariant>()) {
+            // It expects uninitialized memory
+            Q_ALLOCA_VAR(QVariant, result, sizeof(QVariant));
+            const bool isUndefined = !evaluate(result, returnType);
+            if (canWrite())
+                error = !write(result->data(), result->metaType(), isUndefined, flags);
+            result->~QVariant();
+        } else {
+            const auto size = returnType.sizeOf();
+            if (Q_LIKELY(size > 0)) {
+                Q_ALLOCA_VAR(void, result, size);
+                const bool isUndefined = !evaluate(result, returnType);
+                if (canWrite())
+                    error = !write(result, returnType, isUndefined, flags);
+                returnType.destruct(result);
+            } else if (canWrite()) {
+                error = !write(QV4::Encode::undefined(), true, flags);
+            }
+        }
+    } else {
+        bool isUndefined = false;
+        QV4::ScopedValue result(scope, evaluate(&isUndefined));
+        if (canWrite())
+            error = !write(result, isUndefined, flags);
+    }
+
+    if (!watcher.wasDeleted()) {
+
+        if (error) {
+            delayedError()->setErrorLocation(sourceLocation());
+            delayedError()->setErrorObject(m_target.data());
+        }
+
+        if (hasError()) {
+            if (!delayedError()->addError(ep)) ep->warning(this->error(engine()));
+        } else {
+            clearError();
+        }
+    }
+
+    ep->dereferenceScarceResources();
+}
+
+class QObjectPointerBinding: public QQmlBinding
 {
     QQmlMetaObject targetMetaObject;
 
 public:
-    QObjectPointerBinding(QQmlEnginePrivate *engine, int propertyType)
-        : targetMetaObject(QQmlPropertyPrivate::rawMetaObjectForType(engine, propertyType))
+    QObjectPointerBinding(QMetaType propertyType)
+        : targetMetaObject(QQmlPropertyPrivate::rawMetaObjectForType(propertyType))
     {}
 
 protected:
     Q_ALWAYS_INLINE bool write(void *result, QMetaType type, bool isUndefined,
                                QQmlPropertyData::WriteFlags flags) override final
     {
-        QQmlPropertyData *pd;
+        const QQmlPropertyData *pd;
         QQmlPropertyData vtpd;
         getPropertyData(&pd, &vtpd);
         if (Q_UNLIKELY(isUndefined || vtpd.isValid()))
@@ -861,11 +759,8 @@ protected:
             if (resultMo.isNull())
                 resultMo = resultObject->metaObject();
         } else if (type == QMetaType::fromType<QVariant>()) {
-            QVariant value = *static_cast<QVariant *>(result);
-            QQmlEngine *qmlEngine = engine();
-            Q_ASSERT(qmlEngine);
-            resultMo = QQmlPropertyPrivate::rawMetaObjectForType(
-                        QQmlEnginePrivate::get(qmlEngine), value.userType());
+            const QVariant value = *static_cast<QVariant *>(result);
+            resultMo = QQmlPropertyPrivate::rawMetaObjectForType(value.metaType());
             if (resultMo.isNull())
                 return slowWrite(*pd, vtpd, result, type, isUndefined, flags);
             resultObject = *static_cast<QObject *const *>(value.constData());
@@ -881,7 +776,7 @@ protected:
     Q_ALWAYS_INLINE bool write(const QV4::Value &result, bool isUndefined,
                                QQmlPropertyData::WriteFlags flags) override final
     {
-        QQmlPropertyData *pd;
+        const QQmlPropertyData *pd;
         QQmlPropertyData vtpd;
         getPropertyData(&pd, &vtpd);
         if (Q_UNLIKELY(isUndefined || vtpd.isValid()))
@@ -903,11 +798,8 @@ protected:
                 resultMo = resultObject->metaObject();
             }
         } else if (auto variant = result.as<QV4::VariantObject>()) {
-            QVariant value = variant->d()->data();
-            QQmlEngine *qmlEngine = engine();
-            Q_ASSERT(qmlEngine);
-            resultMo = QQmlPropertyPrivate::rawMetaObjectForType(
-                        QQmlEnginePrivate::get(qmlEngine), value.userType());
+            const QVariant value = variant->d()->data();
+            resultMo = QQmlPropertyPrivate::rawMetaObjectForType(value.metaType());
             if (resultMo.isNull())
                 return slowWrite(*pd, vtpd, result, isUndefined, flags);
             resultObject = *static_cast<QObject *const *>(value.constData());
@@ -924,7 +816,7 @@ private:
     using QQmlBinding::slowWrite;
 
     template<typename SlowWrite>
-    bool compareAndSet(const QQmlMetaObject &resultMo, QObject *resultObject, QQmlPropertyData *pd,
+    bool compareAndSet(const QQmlMetaObject &resultMo, QObject *resultObject, const QQmlPropertyData *pd,
                        QQmlPropertyData::WriteFlags flags, const SlowWrite &slowWrite) const
     {
         if (QQmlMetaObject::canConvert(resultMo, targetMetaObject)) {
@@ -940,23 +832,17 @@ private:
     }
 };
 
-QQmlBinding *QQmlBinding::newBinding(QQmlEnginePrivate *engine, const QQmlPropertyData *property)
+QQmlBinding *QQmlBinding::newBinding(const QQmlPropertyData *property)
 {
-    return newBinding(engine, property ? property->propType() : QMetaType());
+    return newBinding(property ? property->propType() : QMetaType());
 }
 
-QQmlBinding *QQmlBinding::newBinding(QQmlEnginePrivate *engine, QMetaType propertyType)
+QQmlBinding *QQmlBinding::newBinding(QMetaType propertyType)
 {
     if (propertyType.flags() & QMetaType::PointerToQObject)
-        return new QObjectPointerBinding(engine, propertyType.id());
+        return new QObjectPointerBinding(propertyType);
 
-    const int type = propertyType.id();
-
-    if (type == qMetaTypeId<QQmlBinding *>()) {
-        return new QQmlBindingBinding;
-    }
-
-    switch (type) {
+    switch (propertyType.id()) {
     case QMetaType::Bool:
         return new GenericBinding<QMetaType::Bool>;
     case QMetaType::Int:
