@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2019 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtQml module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2019 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qqmltype_p_p.h"
 
@@ -72,7 +36,9 @@ QQmlTypePrivate::QQmlTypePrivate(QQmlType::RegistrationType type)
         extraData.cd->attachedPropertiesType = nullptr;
         extraData.cd->propertyValueSourceCast = -1;
         extraData.cd->propertyValueInterceptorCast = -1;
+        extraData.cd->finalizerCast = -1;
         extraData.cd->registerEnumClassesUnscoped = true;
+        extraData.cd->registerEnumsFromRelatedTypes = true;
         break;
     case QQmlType::SingletonType:
     case QQmlType::CompositeSingletonType:
@@ -183,7 +149,8 @@ QQmlType QQmlTypePrivate::resolveCompositeBaseType(QQmlEnginePrivate *engine) co
     return QQmlMetaType::qmlType(mo);
 }
 
-QQmlPropertyCache *QQmlTypePrivate::compositePropertyCache(QQmlEnginePrivate *engine) const
+QQmlPropertyCache::ConstPtr QQmlTypePrivate::compositePropertyCache(
+        QQmlEnginePrivate *engine) const
 {
     // similar logic to resolveCompositeBaseType
     Q_ASSERT(isComposite());
@@ -193,7 +160,7 @@ QQmlPropertyCache *QQmlTypePrivate::compositePropertyCache(QQmlEnginePrivate *en
     if (td.isNull() || !td->isComplete())
         return nullptr;
     QV4::ExecutableCompilationUnit *compilationUnit = td->compilationUnit();
-    return compilationUnit->rootPropertyCache().data();
+    return compilationUnit->rootPropertyCache();
 }
 
 static bool isPropertyRevisioned(const QMetaObject *mo, int index)
@@ -225,11 +192,11 @@ void QQmlTypePrivate::init() const
         QMetaObjectBuilder builder;
         QQmlMetaType::clone(builder, extMetaObject, extMetaObject, extMetaObject,
                             extFunc ? QQmlMetaType::CloneAll : QQmlMetaType::CloneEnumsOnly);
-        builder.setFlags(MetaObjectFlag::DynamicMetaObject);
         QMetaObject *mmo = builder.toMetaObject();
         mmo->d.superdata = mo;
         QQmlProxyMetaObject::ProxyData data = { mmo, extFunc, 0, 0 };
         metaObjects << data;
+        QQmlMetaType::registerMetaObjectForType(mmo, const_cast<QQmlTypePrivate *>(this));
     };
 
     if (regType == QQmlType::SingletonType)
@@ -241,7 +208,7 @@ void QQmlTypePrivate::init() const
             mo, baseMetaObject, metaObjects.isEmpty() ? nullptr
                                                       : metaObjects.constLast().metaObject));
 
-    for (int ii = 0; ii < metaObjects.count(); ++ii) {
+    for (int ii = 0; ii < metaObjects.size(); ++ii) {
         metaObjects[ii].propertyOffset =
                 metaObjects.at(ii).metaObject->propertyOffset();
         metaObjects[ii].methodOffset =
@@ -273,9 +240,9 @@ void QQmlTypePrivate::init() const
 
 void QQmlTypePrivate::initEnums(QQmlEnginePrivate *engine) const
 {
-    const QQmlPropertyCache *cache = (!isEnumFromCacheSetup.loadAcquire() && isComposite())
+    QQmlPropertyCache::ConstPtr cache = (!isEnumFromCacheSetup.loadAcquire() && isComposite())
             ? compositePropertyCache(engine)
-            : nullptr;
+            : QQmlPropertyCache::ConstPtr();
 
     // beware: It could be a singleton type without metaobject
     const QMetaObject *metaObject = !isEnumFromBaseSetup.loadAcquire()
@@ -303,11 +270,12 @@ void QQmlTypePrivate::initEnums(QQmlEnginePrivate *engine) const
 void QQmlTypePrivate::insertEnums(const QMetaObject *metaObject) const
 {
     // Add any enum values defined by 'related' classes
-    if (metaObject->d.relatedMetaObjects) {
-        const auto *related = metaObject->d.relatedMetaObjects;
-        if (related) {
-            while (*related)
-                insertEnums(*related++);
+    if (regType != QQmlType::CppType || extraData.cd->registerEnumsFromRelatedTypes) {
+        if (const auto *related = metaObject->d.relatedMetaObjects) {
+            while (const QMetaObject *relatedMetaObject = *related) {
+                insertEnums(relatedMetaObject);
+                ++related;
+            }
         }
     }
 
@@ -352,7 +320,7 @@ void QQmlTypePrivate::insertEnums(const QMetaObject *metaObject) const
 
         if (isScoped) {
             scopedEnums << scoped;
-            scopedEnumIndex.insert(QString::fromUtf8(e.name()), scopedEnums.count()-1);
+            scopedEnumIndex.insert(QString::fromUtf8(e.name()), scopedEnums.size()-1);
         }
     }
 }
@@ -403,33 +371,35 @@ void QQmlTypePrivate::createEnumConflictReport(const QMetaObject *metaObject, co
 
     qWarning().noquote() << QLatin1String("Possible conflicting items:");
     // find items with conflicting key
-    for (const auto &i : qAsConst(enumInfoList)) {
+    for (const auto &i : std::as_const(enumInfoList)) {
         if (i.enumKey == conflictingKey)
         qWarning().noquote().nospace() << "    " << i.metaObjectName << "." << i.enumName << "." << i.enumKey << " from scope "
                                            << i.metaEnumScope << " injected by " << i.path.join(QLatin1String("->"));
     }
 }
 
-void QQmlTypePrivate::insertEnumsFromPropertyCache(const QQmlPropertyCache *cache) const
+void QQmlTypePrivate::insertEnumsFromPropertyCache(
+        const QQmlPropertyCache::ConstPtr &cache) const
 {
     const QMetaObject *cppMetaObject = cache->firstCppMetaObject();
 
-    while (cache && cache->metaObject() != cppMetaObject) {
+    for (const QQmlPropertyCache *currentCache = cache.data();
+         currentCache && currentCache->metaObject() != cppMetaObject;
+         currentCache = currentCache->parent().data()) {
 
-        int count = cache->qmlEnumCount();
+        int count = currentCache->qmlEnumCount();
         for (int ii = 0; ii < count; ++ii) {
             QStringHash<int> *scoped = new QStringHash<int>();
-            QQmlEnumData *enumData = cache->qmlEnum(ii);
+            QQmlEnumData *enumData = currentCache->qmlEnum(ii);
 
-            for (int jj = 0; jj < enumData->values.count(); ++jj) {
+            for (int jj = 0; jj < enumData->values.size(); ++jj) {
                 const QQmlEnumValue &value = enumData->values.at(jj);
                 enums.insert(value.namedValue, value.value);
                 scoped->insert(value.namedValue, value.value);
             }
             scopedEnums << scoped;
-            scopedEnumIndex.insert(enumData->name, scopedEnums.count()-1);
+            scopedEnumIndex.insert(enumData->name, scopedEnums.size()-1);
         }
-        cache = cache->parent();
     }
     insertEnums(cppMetaObject);
 }
@@ -474,34 +444,61 @@ QString QQmlType::qmlTypeName() const
     return d->name;
 }
 
+/*!
+   \internal
+   Allocates and initializes an object if the type is creatable.
+   Returns a pointer to the object, or nullptr if the type was
+   not creatable.
+ */
 QObject *QQmlType::create() const
+{
+    void *unused;
+    return create(&unused, 0);
+}
+
+/*!
+   \internal
+   \brief Like create without arguments, but allocates some extra space after the object.
+   \param memory An out-only argument. *memory will point to the start of the additionally
+                 allocated memory.
+   \param additionalMemory The amount of extra memory in bytes that shoudld be allocated.
+
+   \note This function is used to allocate the QQmlData next to the object in the
+   QQmlObjectCreator.
+
+   \overload
+ */
+QObject *QQmlType::create(void **memory, size_t additionalMemory) const
 {
     if (!d || !isCreatable())
         return nullptr;
 
     d->init();
 
-    QObject *rv = (QObject *)operator new(d->extraData.cd->allocationSize);
-
-    d->extraData.cd->newFunc(rv, d->extraData.cd->userdata);
-
-    createProxy(rv);
-    return rv;
-}
-
-void QQmlType::create(QObject **out, void **memory, size_t additionalMemory) const
-{
-    if (!d || !isCreatable())
-        return;
-
-    d->init();
-
     QObject *rv = (QObject *)operator new(d->extraData.cd->allocationSize + additionalMemory);
     d->extraData.cd->newFunc(rv, d->extraData.cd->userdata);
 
     createProxy(rv);
-    *out = rv;
     *memory = ((char *)rv) + d->extraData.cd->allocationSize;
+    return rv;
+}
+
+/*!
+    \internal
+    Like create, but also allocates memory behind the object, constructs a QQmlData there
+    and lets the objects declarativeData point to the newly created QQmlData.
+ */
+QObject *QQmlType::createWithQQmlData() const
+{
+    void *ddataMemory = nullptr;
+    auto instance = create(&ddataMemory, sizeof(QQmlData));
+    if (!instance)
+        return nullptr;
+    QObjectPrivate* p = QObjectPrivate::get(instance);
+    Q_ASSERT(!p->isDeletingChildren);
+    if (!p->declarativeData)
+        p->declarativeData = new (ddataMemory) QQmlData(QQmlData::DoesNotOwnMemory);
+    return instance;
 }
 
 QQmlType::SingletonInstanceInfo *QQmlType::singletonInstanceInfo() const
@@ -529,6 +526,20 @@ QQmlType::CreateValueTypeFunc QQmlType::createValueTypeFunction() const
     return d->extraData.cd->createValueTypeFunc;
 }
 
+bool QQmlType::canConstructValueType() const
+{
+    if (!d || d->regType != CppType)
+        return false;
+    return d->extraData.cd->constructValueType;
+}
+
+bool QQmlType::canPopulateValueType() const
+{
+    if (!d || d->regType != CppType)
+        return false;
+    return d->extraData.cd->populateValueType;
+}
+
 QQmlType::CreateFunc QQmlType::createFunction() const
 {
     if (!d || d->regType != CppType)
@@ -636,6 +647,11 @@ QMetaType QQmlType::qListTypeId() const
     return d ? d->listId : QMetaType{};
 }
 
+QMetaSequence QQmlType::listMetaSequence() const
+{
+    return isSequentialContainer() ? *d->extraData.ld : QMetaSequence();
+}
+
 const QMetaObject *QQmlType::metaObject() const
 {
     if (!d)
@@ -703,6 +719,13 @@ int QQmlType::propertyValueInterceptorCast() const
     return d->extraData.cd->propertyValueInterceptorCast;
 }
 
+int QQmlType::finalizerCast() const
+{
+    if (!d || d->regType != CppType)
+        return -1;
+    return d->extraData.cd->finalizerCast;
+}
+
 const char *QQmlType::interfaceIId() const
 {
     if (!d || d->regType != InterfaceType)
@@ -831,7 +854,7 @@ int QQmlType::scopedEnumValue(QQmlEnginePrivate *engine, int index, const QV4::S
     *ok = true;
 
     if (d) {
-        Q_ASSERT(index > -1 && index < d->scopedEnums.count());
+        Q_ASSERT(index > -1 && index < d->scopedEnums.size());
         int *rv = d->scopedEnums.at(index)->value(name);
         if (rv)
             return *rv;
@@ -848,7 +871,7 @@ int QQmlType::scopedEnumValue(QQmlEnginePrivate *engine, int index, const QStrin
     *ok = true;
 
     if (d) {
-        Q_ASSERT(index > -1 && index < d->scopedEnums.count());
+        Q_ASSERT(index > -1 && index < d->scopedEnums.size());
         int *rv = d->scopedEnums.at(index)->value(name);
         if (rv)
             return *rv;
@@ -866,11 +889,11 @@ int QQmlType::scopedEnumValue(QQmlEnginePrivate *engine, const QByteArray &scope
 
         d->initEnums(engine);
 
-        int *rv = d->scopedEnumIndex.value(QHashedCStringRef(scopedEnumName.constData(), scopedEnumName.length()));
+        int *rv = d->scopedEnumIndex.value(QHashedCStringRef(scopedEnumName.constData(), scopedEnumName.size()));
         if (rv) {
             int index = *rv;
-            Q_ASSERT(index > -1 && index < d->scopedEnums.count());
-            rv = d->scopedEnums.at(index)->value(QHashedCStringRef(name.constData(), name.length()));
+            Q_ASSERT(index > -1 && index < d->scopedEnums.size());
+            rv = d->scopedEnums.at(index)->value(QHashedCStringRef(name.constData(), name.size()));
             if (rv)
                 return *rv;
         }
@@ -891,7 +914,7 @@ int QQmlType::scopedEnumValue(QQmlEnginePrivate *engine, QStringView scopedEnumN
         int *rv = d->scopedEnumIndex.value(QHashedStringRef(scopedEnumName));
         if (rv) {
             int index = *rv;
-            Q_ASSERT(index > -1 && index < d->scopedEnums.count());
+            Q_ASSERT(index > -1 && index < d->scopedEnums.size());
             rv = d->scopedEnums.at(index)->value(QHashedStringRef(name));
             if (rv)
                 return *rv;
