@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 #include <qtest.h>
 #include <QDebug>
 
@@ -38,14 +13,41 @@
 #include <QtQuick/private/qquickpalette_p.h>
 #include <QtQuickTestUtils/private/qmlutils_p.h>
 #include <QtQuickTestUtils/private/testhttpserver_p.h>
+#include <private/qqmlcomponent_p.h>
 #include <private/qqmlguardedcontextdata_p.h>
 #include <private/qv4qmlcontext_p.h>
-#include <private/qv4scopedvalue_p.h>
 #include <private/qv4qmlcontext_p.h>
+#include <private/qv4scopedvalue_p.h>
+#include <private/qv4executablecompilationunit_p.h>
 #include <qcolor.h>
-
+#include <qsignalspy.h>
+#include "lifecyclewatcher.h"
 #include <algorithm>
 
+using namespace Qt::StringLiterals;
+
+class WithQJSValue : public QObject
+{
+    Q_OBJECT
+    Q_PROPERTY(QJSValue v READ v WRITE setV NOTIFY vChanged)
+
+public:
+    QJSValue v() const { return m_v; }
+    void setV(const QJSValue &newV)
+    {
+        if (!m_v.strictlyEquals(newV)) {
+            m_v = newV;
+            emit vChanged();
+        }
+    }
+
+signals:
+    void vChanged();
+
+private:
+    QJSValue m_v;
+};
+
 class MyIC : public QObject, public QQmlIncubationController
 {
     Q_OBJECT
@@ -108,6 +110,8 @@ private slots:
     void qmlCreateObjectAutoParent_data();
     void qmlCreateObjectAutoParent();
     void qmlCreateObjectWithProperties();
+    void qmlCreateObjectClean();
+    void qmlCreateObjectDirty();
     void qmlIncubateObject();
     void qmlCreateParentReference();
     void async();
@@ -130,7 +134,21 @@ private slots:
     void testSetInitialProperties();
     void createInsideJSModule();
     void qmlErrorIsReported();
+    void initJSValueProp();
+    void qmlPropertySignalExists();
+    void componentTypes();
+    void boundComponent();
+    void loadFromModule_data();
+    void loadFromModule();
+    void loadFromModuleLifecycle();
+    void loadFromModuleThenCreateWithIncubator();
+    void loadFromModuleFailures_data();
+    void loadFromModuleFailures();
+    void loadFromModuleRequired();
+    void loadFromQrc();
     void removeBinding();
+    void complexObjectArgument();
+    void bindingInRequired();
 
 private:
     QQmlEngine engine;
@@ -157,7 +175,7 @@ void tst_qqmlcomponent::loadEmptyUrl()
     c.loadUrl(QUrl());
 
     QVERIFY(c.isError());
-    QCOMPARE(c.errors().count(), 1);
+    QCOMPARE(c.errors().size(), 1);
     QQmlError error = c.errors().first();
     QCOMPARE(error.url(), QUrl());
     QCOMPARE(error.line(), -1);
@@ -183,6 +201,7 @@ void tst_qqmlcomponent::qmlCreateWindow()
     QQmlEngine engine;
     QQmlComponent component(&engine);
     component.loadUrl(testFileUrl("createWindow.qml"));
+    QVERIFY2(component.isReady(), qPrintable(component.errorString()));
     QScopedPointer<QQuickWindow> window(qobject_cast<QQuickWindow *>(component.create()));
     QVERIFY(!window.isNull());
 }
@@ -259,6 +278,16 @@ void tst_qqmlcomponent::qmlCreateObjectAutoParent()
 void tst_qqmlcomponent::qmlCreateObjectWithProperties()
 {
     QQmlEngine engine;
+
+    QTest::ignoreMessage(
+            QtMsgType::QtWarningMsg,
+            QRegularExpression(".*createObjectWithScript.qml: Setting initial properties failed: "
+                               "Item does not have a property called not_i"));
+    QTest::ignoreMessage(
+            QtMsgType::QtWarningMsg,
+            QRegularExpression(
+                    ".*createObjectWithScript.qml:45:13: Required property i was not initialized"));
+
     QQmlComponent component(&engine, testFileUrl("createObjectWithScript.qml"));
     QVERIFY2(component.errorString().isEmpty(), component.errorString().toUtf8());
     QScopedPointer<QObject> object(component.create());
@@ -307,6 +336,58 @@ void tst_qqmlcomponent::qmlCreateObjectWithProperties()
         testBindingThisObj->setProperty("width", 200);
         QCOMPARE(testBindingThisObj->property("testValue").value<int>(), 200 * 3);
     }
+
+    {
+        QScopedPointer<QObject> badRequired(object->property("badRequired").value<QObject *>());
+        QVERIFY(!badRequired);
+
+        QScopedPointer<QObject> goodRequired(object->property("goodRequired").value<QObject *>());
+        QVERIFY(goodRequired);
+        QCOMPARE(goodRequired->parent(), object.data());
+        QCOMPARE(goodRequired->property("i").value<int>(), 42);
+    }
+
+    {
+        QScopedPointer<QObject> bindingAsInitial(object->property("bindingAsInitial").value<QObject *>());
+        QVERIFY(bindingAsInitial);
+        QVERIFY(object->property("bindingUsed").toBool());
+    }
+}
+
+void tst_qqmlcomponent::qmlCreateObjectClean()
+{
+    QQmlEngine engine;
+    QVERIFY(engine.outputWarningsToStandardError());
+    QObject::connect(&engine, &QQmlEngine::warnings, [](const QList<QQmlError> &) {
+        QFAIL("Calls with suitable parameters should not generate any warnings.");
+    });
+    QQmlComponent component(&engine, testFileUrl("createObjectClean.qml"));
+    QVERIFY2(component.errorString().isEmpty(), component.errorString().toUtf8());
+    QScopedPointer<QObject> object(component.create());
+    QVERIFY(!object.isNull());
+
+    QVERIFY(qvariant_cast<QObject *>(object->property("a")) != nullptr);
+    QVERIFY(qvariant_cast<QObject *>(object->property("b")) != nullptr);
+    QVERIFY(qvariant_cast<QObject *>(object->property("c")) != nullptr);
+    QVERIFY(qvariant_cast<QObject *>(object->property("d")) != nullptr);
+}
+
+void tst_qqmlcomponent::qmlCreateObjectDirty()
+{
+    QQmlEngine engine;
+    engine.setOutputWarningsToStandardError(false);
+    QObject::connect(&engine, &QQmlEngine::warnings, [](const QList<QQmlError> &warnings) {
+        QCOMPARE(warnings.size(), 1);
+        QCOMPARE(warnings[0].description(),
+                "QML Component: Unsuitable arguments passed to createObject(). The first argument "
+                "should be a QObject* or null, and the second argument should be a JavaScript "
+                "object or a QVariantMap");
+    });
+    QQmlComponent component(&engine, testFileUrl("createObjectDirty.qml"));
+    QVERIFY2(component.errorString().isEmpty(), component.errorString().toUtf8());
+    QScopedPointer<QObject> object(component.create());
+    QVERIFY(!object.isNull());
+    QVERIFY(qvariant_cast<QObject *>(object->property("a")) != nullptr);
 }
 
 void tst_qqmlcomponent::qmlCreateParentReference()
@@ -514,13 +595,14 @@ void tst_qqmlcomponent::onDestructionCount()
     engine.setOutputWarningsToStandardError(false);
     QCOMPARE(engine.outputWarningsToStandardError(), false);
 
-    QCOMPARE(warnings.count(), 0);
+    QCOMPARE(warnings.size(), 0);
 }
 
 void tst_qqmlcomponent::recursion()
 {
     QQmlEngine engine;
     QQmlComponent component(&engine, testFileUrl("recursion.qml"));
+    QVERIFY2(component.isReady(), qPrintable(component.errorString()));
 
     QTest::ignoreMessage(QtWarningMsg, QLatin1String("QQmlComponent: Component creation is recursing - aborting").data());
     QScopedPointer<QObject> object(component.create());
@@ -872,12 +954,13 @@ void tst_qqmlcomponent::testRequiredProperties()
                  "We fail to recognized required sub-properties inside a group property, even when "
                  "that group property is used (QTBUG-96544)",
                  Abort);
-    if (shouldSucceed)
+    if (shouldSucceed) {
+        QVERIFY2(comp.isReady(), qPrintable(comp.errorString()));
         QVERIFY(obj);
-    else {
+    } else {
         QVERIFY2(!obj, "The object is valid when it shouldn't be");
         QFETCH(QString, errorMsg);
-        QVERIFY(comp.errorString().contains(errorMsg));
+        QVERIFY2(comp.errorString().contains(errorMsg), qPrintable(comp.errorString()));
     }
 }
 
@@ -981,11 +1064,64 @@ void tst_qqmlcomponent::testSetInitialProperties()
         // createWithInitialProperties: setting a nonexistent property
         QQmlComponent comp(&eng);
         comp.loadUrl(testFileUrl("allJSONTypes.qml"));
+        const QRegularExpression errorMessage { QStringLiteral(
+                ".*allJSONTypes.qml: Setting initial properties failed: Item does not have a "
+                "property called notThePropertiesYoureLookingFor") };
+        QTest::ignoreMessage(QtMsgType::QtWarningMsg, errorMessage);
         QScopedPointer<QObject> obj {
             comp.createWithInitialProperties(QVariantMap { {"notThePropertiesYoureLookingFor", 42} })
         };
         QVERIFY(obj);
-        QVERIFY(comp.errorString().contains("Setting initial properties failed: Item does not have a property called notThePropertiesYoureLookingFor"));
+        QVERIFY(comp.isReady()); // despite the error, the component is still ready
+
+        // QTBUG-101439: repeated creation succeeds as well
+        QScopedPointer<QObject> objEmpty { comp.create() };
+        QVERIFY(objEmpty);
+    }
+
+    {
+        QQmlComponent comp(&eng);
+        comp.loadUrl(testFileUrl("requiredNotSet.qml"));
+        QTest::ignoreMessage(
+                QtMsgType::QtWarningMsg,
+                QRegularExpression(".*requiredNotSet.qml: Setting initial properties failed: Item "
+                                   "does not have a property called not_i"));
+        QScopedPointer<QObject> obj { comp.createWithInitialProperties(
+                QVariantMap { { QLatin1String("not_i"), QJsonValue { 42 } } }) };
+        QVERIFY(!obj);
+        QVERIFY(comp.isError());
+        QVERIFY(comp.errorString().contains("Required property i was not initialized"));
+
+        QScopedPointer<QObject> objGood { comp.createWithInitialProperties(
+                QVariantMap { { QLatin1String("i"), QJsonValue { 42 } } }) };
+        QVERIFY2(objGood, qPrintable(comp.errorString()));
+        QCOMPARE(objGood->property("i"), 42);
+    }
+
+    // QJSValue unpacking - QTBUG-101440
+    {
+        QQmlComponent comp(&eng);
+        comp.setData(R"(
+            import QtQml
+            QtObject {
+                property int x
+                property int y: func ? func() : -1
+                property var func // special
+            }
+        )", QUrl());
+
+        QJSValue data = eng.evaluate("({ \"x\": 42, \"func\": (function() { return 42; }) })");
+        QVERIFY(data.isObject());
+        QVariant var = data.toVariant();
+        QCOMPARE(var.typeId(), QMetaType::QVariantMap);
+        QVariantMap properties = var.toMap();
+        QScopedPointer<QObject> object { comp.createWithInitialProperties(properties) };
+        QVERIFY(object);
+        QCOMPARE(object->property("x"), 42);
+        QCOMPARE(object->property("y"), 42);
+        QJSValue func = eng.toScriptValue(object->property("func"));
+        QVERIFY(func.isCallable());
+        QCOMPARE(func.call().toInt(), 42);
     }
 }
 
@@ -1023,14 +1159,370 @@ void tst_qqmlcomponent::qmlErrorIsReported()
     }));
 }
 
+void tst_qqmlcomponent::initJSValueProp()
+{
+    qmlRegisterType<WithQJSValue>("ComponentTest", 1, 0, "WithQJSValue");
+    QQmlEngine engine;
+    QQmlComponent component(&engine);
+    component.setData("import ComponentTest\nWithQJSValue {}", QUrl());
+    QVERIFY2(component.isReady(), qPrintable(component.errorString()));
+    QScopedPointer<QObject> o(component.createWithInitialProperties({{ u"v"_s, 5}}));
+    QVERIFY(!o.isNull());
+    WithQJSValue *withQJSValue = qobject_cast<WithQJSValue *>(o.data());
+    QVERIFY(withQJSValue);
+    const QJSValue jsValue = withQJSValue->v();
+    QVERIFY(jsValue.isNumber());
+    QCOMPARE(jsValue.toInt(), 5);
+}
+
+void tst_qqmlcomponent::qmlPropertySignalExists()
+{
+    QQmlEngine engine;
+    QQmlComponent component(&engine);
+    component.setData("import QtQml; QtObject { property int p: 41; function doStuff() { p++; } }",
+                      QUrl());
+    QVERIFY2(component.isReady(), qPrintable(component.errorString()));
+    QScopedPointer<QObject> o(component.create());
+    QVERIFY(!o.isNull());
+
+    QSignalSpy changeSignalSpy(o.get(), SIGNAL(pChanged()));
+    QVERIFY(QMetaObject::invokeMethod(o.get(), "doStuff"));
+    QCOMPARE(changeSignalSpy.size(), 1);
+    QCOMPARE(o->property("p").toInt(), 42);
+}
+
+void tst_qqmlcomponent::componentTypes()
+{
+    {
+        QQmlEngine engine;
+        QQmlComponent component(&engine);
+        // not allowed: "Cannot create empty component specification"
+        component.setData("import QtQml; Component { }", QUrl());
+        QVERIFY(!component.isReady());
+    }
+
+    {
+        QQmlEngine engine;
+        QQmlComponent component(&engine);
+        component.loadUrl(testFileUrl("ComponentType.qml"));
+        QScopedPointer<QObject> o(component.create());
+        QVERIFY2(!o.isNull(), qPrintable(component.errorString()));
+        QQmlComponent *oComponent = qobject_cast<QQmlComponent *>(o.get());
+        QVERIFY(oComponent);
+        QScopedPointer<QObject> enclosed(oComponent->create());
+        QVERIFY(!enclosed.isNull());
+        QCOMPARE(enclosed->objectName(), u"enclosed"_s);
+    }
+
+    {
+        QQmlEngine engine;
+        QQmlComponent component(&engine);
+        component.loadUrl(testFileUrl("componentTypes.qml"));
+        QScopedPointer<QObject> o(component.create());
+        QVERIFY2(!o.isNull(), qPrintable(component.errorString()));
+
+        QQmlContext *ctx = engine.contextForObject(o.get());
+
+        QObject *normal = ctx->objectForName(u"normal"_s);
+        QVERIFY(normal);
+        QCOMPARE(normal->property("text").toString(), u"indirect component"_s);
+
+        // check (and thus "document" in code) various ways of how ids work
+        QVERIFY(ctx->objectForName(u"accessibleNormal"_s));
+        QVERIFY(!ctx->objectForName(u"inaccessibleNormal"_s));
+        QVERIFY(ctx->objectForName(u"accessible"_s));
+        QVERIFY(!ctx->objectForName(u"inaccessible"_s));
+        QVERIFY(ctx->objectForName(u"accessibleDelegate"_s));
+        QVERIFY(!ctx->objectForName(u"inaccessibleDelegate"_s));
+
+        QCOMPARE(qvariant_cast<QObject *>(o->property("p2"))->property("text").toString(),
+                 u"foo"_s);
+        auto p3Object = qvariant_cast<QObject *>(o->property("p3"));
+        QVERIFY(p3Object);
+        QVERIFY(p3Object->property("text").toString().isEmpty());
+
+        QQmlComponent *normalComponent = qobject_cast<QQmlComponent *>(normal);
+        QVERIFY(normalComponent);
+        QScopedPointer<QObject> enclosed(normalComponent->create());
+        QVERIFY(enclosed);
+        QCOMPARE(enclosed->objectName(), u"enclosed"_s);
+    }
+}
+
+void tst_qqmlcomponent::boundComponent()
+{
+    QQmlEngine engine;
+    {
+        QQmlComponent component(&engine, testFileUrl("nestedBoundComponent.qml"));
+        QVERIFY2(component.isReady(), qPrintable(component.errorString()));
+        QVERIFY(component.isBound());
+
+        QScopedPointer<QObject> o(component.create());
+        QVERIFY(!o.isNull());
+
+        QQmlComponent *nestedComponent = o->property("c").value<QQmlComponent *>();
+        QVERIFY(nestedComponent != nullptr);
+        QVERIFY(nestedComponent->isBound());
+
+        QObject *nestedObject = o->property("o").value<QObject *>();
+        QVERIFY(nestedObject != nullptr);
+        QCOMPARE(nestedObject->objectName(), QLatin1String("bound"));
+
+        QScopedPointer<QObject> contextedObject(nestedComponent->create(qmlContext(o.data())));
+        QVERIFY(!contextedObject.isNull());
+
+        QScopedPointer<QObject> uncontextedObject(nestedComponent->create());
+        QVERIFY(uncontextedObject.isNull());
+
+        QVERIFY(nestedComponent->errorString().contains(
+                QLatin1String("Cannot instantiate bound component outside its creation context")));
+    }
+
+    {
+        QQmlComponent component(&engine, testFileUrl("BoundInlineComponent.qml"));
+        QVERIFY2(component.isReady(), qPrintable(component.errorString()));
+        QVERIFY(component.isBound());
+
+        QScopedPointer<QObject> o(component.create());
+        QVERIFY2(!o.isNull(), qPrintable(component.errorString()));
+
+        QObject *nestedObject = o->property("o").value<QObject *>();
+        QVERIFY(nestedObject != nullptr);
+
+        QCOMPARE(nestedObject->objectName(), QLatin1String("inline"));
+    }
+
+    {
+        QQmlComponent component(&engine, testFileUrl("boundInlineComponentUser.qml"));
+        QVERIFY2(component.isReady(), qPrintable(component.errorString()));
+        QVERIFY(!component.isBound());
+        QScopedPointer<QObject> o(component.create());
+        QVERIFY(o.isNull());
+
+        QVERIFY(component.errorString().contains(
+                QLatin1String("Cannot instantiate bound inline component in different file")));
+
+    }
+
+    {
+        QQmlComponent component(&engine);
+        QVERIFY(!component.isBound());
+
+        component.setData("pragma ComponentBehavior: Bound\nsyntax error", QUrl());
+        QCOMPARE(component.errorString(), ":2 Syntax error\n"_L1);
+        QVERIFY(!component.isBound());
+    }
+}
+
+void tst_qqmlcomponent::loadFromModule_data()
+{
+    using namespace Qt::StringLiterals;
+
+    QTest::addColumn<QString>("uri");
+    QTest::addColumn<QString>("typeName");
+    QTest::addColumn<QString>("classNameRe");
+
+    QTest::addRow("Item") << u"QtQuick"_s << u"Item"_s << u"QQuickItem"_s;
+#if defined(HAS_CONTROLS)
+    QTest::addRow("Button") << u"QtQuick.Controls"_s << u"Button"_s << u"Button_QMLTYPE_\\d+"_s;
+    QTest::addRow("Basic.Button") << u"QtQuick.Controls.Basic"_s << u"Button"_s << u"Button_QMLTYPE_\\d+"_s;
+#endif
+
+    QTest::addRow("IC") << u"test"_s << u"TestComponentWithIC"_s << u"TestComponentWithIC"_s; // sanity check for next test
+    QTest::addRow("IC") << u"test"_s << u"TestComponentWithIC.InnerIC"_s << u"InnerIC"_s;
+
+    QTest::addRow("plainQML") << u"plainqml"_s << u"Plain"_s << u"Plain"_s;
+}
+
+void tst_qqmlcomponent::loadFromModule()
+{
+    QFETCH(QString, uri);
+    QFETCH(QString, typeName);
+    QFETCH(QString, classNameRe);
+    QQmlEngine engine;
+    QQmlComponent component(&engine);
+    QCOMPARE(component.progress(), 0);
+    QSignalSpy progressSpy(&component, &QQmlComponent::progressChanged);
+    component.loadFromModule(uri, typeName);
+    QVERIFY2(component.isReady(), qPrintable(component.errorString()));
+
+    // verify that we changed the progress correctly to 1
+    QVERIFY(!progressSpy.isEmpty() || progressSpy.wait());
+    QCOMPARE(progressSpy.last().at(0).toDouble(), 1.0);
+
+    std::unique_ptr<QObject> object(component.create());
+    QVERIFY(object);
+    QRegularExpression classNameMatcher(classNameRe);
+    const char *name = object->metaObject()->className();
+    QVERIFY2(classNameMatcher.match(name).hasMatch(),
+             name);
+}
+
+void tst_qqmlcomponent::loadFromModuleLifecycle()
+{
+    QQmlEngine engine;
+    QList<int> loadFromModuleOrder;
+    QList<int> plainLoadOrder;
+    const QList<int> expected {1, 2, 3};
+    {
+        QQmlComponent component(&engine);
+        component.loadFromModule("test", "LifeCycleWatcher");
+        QVERIFY2(component.isReady(), qPrintable(component.errorString()));
+        std::unique_ptr<QObject> root{ component.create() };
+        LifeCycleWatcher *watcher = qobject_cast<LifeCycleWatcher *>(root.get());
+        QVERIFY(watcher);
+        loadFromModuleOrder = watcher->states;
+        QCOMPARE(loadFromModuleOrder, expected);
+    }
+    {
+        QQmlComponent component(&engine);
+        component.setData("import test; LifeCycleWatcher {}", {});
+        QVERIFY2(component.isReady(), qPrintable(component.errorString()));
+        std::unique_ptr<QObject> root{ component.create() };
+        LifeCycleWatcher *watcher = qobject_cast<LifeCycleWatcher *>(root.get());
+        QVERIFY(watcher);
+        plainLoadOrder = watcher->states;
+    }
+    QCOMPARE(loadFromModuleOrder, plainLoadOrder);
+}
+
+struct CallVerifyingIncubtor : QQmlIncubator
+{
+    void setInitialState(QObject *) override { setInitialStateCalled = true; }
+    void statusChanged(QQmlIncubator::Status status) override { lastStatus = status; }
+
+    QQmlIncubator::Status lastStatus = QQmlIncubator::Null;
+    bool setInitialStateCalled = false;
+};
+
+void tst_qqmlcomponent::loadFromModuleThenCreateWithIncubator()
+{
+    QQmlEngine engine;
+    QQmlComponent comp(&engine);
+    comp.loadFromModule("QtQuick", "Rectangle");
+    CallVerifyingIncubtor incubator;
+    comp.create(incubator);
+    std::unique_ptr<QObject> object { incubator.object() };
+    QVERIFY(incubator.setInitialStateCalled);
+    QVERIFY(incubator.isReady());
+    QCOMPARE(incubator.lastStatus, QQmlIncubator::Ready);
+    QCOMPARE(object->metaObject()->className(), "QQuickRectangle");
+}
+
+void tst_qqmlcomponent::loadFromModuleFailures_data()
+{
+
+    QTest::addColumn<QString>("uri");
+    QTest::addColumn<QString>("typeName");
+    QTest::addColumn<QString>("errorMsg");
+
+    QTest::addRow("noSuchModule") << "Does.Not.Exist"
+                                  << "Type"
+                                  << "No module named \"Does.Not.Exist\" found";
+    QTest::addRow("noSuchType") << "QtQml"
+                                << "NoSuchType"
+                                << "Module \"QtQml\" contains no type named \"NoSuchType\"";
+    QTest::addRow("CppSingleton") << u"QtQuick"_s
+                                  << u"Application"_s
+                                  << u"Application is a singleton, and cannot be loaded"_s;
+}
+
+void tst_qqmlcomponent::loadFromModuleFailures()
+{
+    QFETCH(QString, uri);
+    QFETCH(QString, typeName);
+    QFETCH(QString, errorMsg);
+
+    QQmlEngine engine;
+    QQmlComponent component(&engine);
+    QSignalSpy errorSpy(&component, &QQmlComponent::statusChanged);
+    component.loadFromModule(uri, typeName);
+    QVERIFY(!errorSpy.isEmpty());
+    QCOMPARE(errorSpy.first().first().value<QQmlComponent::Status>(),
+             QQmlComponent::Error);
+    QVERIFY(!component.errors().isEmpty());
+    QCOMPARE(component.errors().constFirst().description(),
+             errorMsg);
+}
+
+struct SingleRequiredProperty : QObject
+{
+    Q_OBJECT
+    Q_PROPERTY(int i MEMBER i REQUIRED)
+
+    int i = 42;
+};
+
+void tst_qqmlcomponent::loadFromModuleRequired()
+{
+
+    QQmlEngine engine;
+    qmlRegisterType<SingleRequiredProperty>("qqmlcomponenttest", 1, 0, "SingleRequiredProperty");
+    QQmlComponent component(&engine, "qqmlcomponenttest", "SingleRequiredProperty");
+    QVERIFY2(!component.isError(), qPrintable(component.errorString()));
+
+    QScopedPointer<QObject> root(component.create());
+    QVERIFY(!root);
+}
+
+void tst_qqmlcomponent::loadFromQrc()
+{
+    QQmlEngine engine;
+    QQmlComponent component(&engine, QStringLiteral(":/qt/qml/test/data/withAot.qml"));
+    QVERIFY2(component.isReady(), qPrintable(component.errorString()));
+
+    QQmlComponentPrivate *p = QQmlComponentPrivate::get(&component);
+    QVERIFY(p);
+    QVERIFY(p->compilationUnit);
+    QVERIFY(p->compilationUnit->aotCompiledFunctions);
+}
+
 void tst_qqmlcomponent::removeBinding()
 {
     QQmlEngine e;
-    QQmlComponent c(&e, testFileUrl("removeBinding.qml"));
+    const QUrl url = testFileUrl("removeBinding.qml");
+    QQmlComponent c(&e, url);
     QVERIFY2(c.isReady(), qPrintable(c.errorString()));
+
+    QTest::ignoreMessage(
+        QtWarningMsg,
+        qPrintable(url.toString() + QStringLiteral(":7:27: QML Component: Unsuitable arguments "
+                                                   "passed to createObject(). The first argument "
+                                                   "should be a QObject* or null, and the second "
+                                                   "argument should be a JavaScript object or a "
+                                                   "QVariantMap")));
     QScopedPointer<QObject> o(c.create());
     QVERIFY(!o.isNull());
     QCOMPARE(o->property("result"), QStringLiteral("42"));
+    QCOMPARE(o->property("result2"), QStringLiteral("43"));
+}
+
+void tst_qqmlcomponent::complexObjectArgument()
+{
+    QQmlEngine e;
+    QQmlComponent c(&e, testFileUrl("complexObjectArgument.qml"));
+    QVERIFY2(c.isReady(), qPrintable(c.errorString()));
+    QScopedPointer<QObject> o(c.create());
+    QVERIFY(!o.isNull());
+    QCOMPARE(o->objectName(), QStringLiteral("26 - 25"));
+}
+
+void tst_qqmlcomponent::bindingInRequired()
+{
+    QQmlEngine engine;
+    QQmlComponent component(&engine, testFileUrl("bindingInRequired.qml"));
+    QVERIFY2(component.isReady(), qPrintable(component.errorString()));
+    QScopedPointer<QObject> object(component.create());
+    QVERIFY(!object.isNull());
+
+    QObject *outer = object->property("outer").value<QObject *>();
+    QVERIFY(outer);
+
+    QObject *inner = object->property("inner").value<QObject *>();
+    QVERIFY(inner);
+
+    QCOMPARE(inner, outer->property("obj").value<QObject *>());
+    QVERIFY(!inner->property("obj").value<QObject *>());
 }
 
 QTEST_MAIN(tst_qqmlcomponent)
