@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include <qtest.h>
 #include <QDebug>
@@ -74,8 +49,10 @@ private slots:
     void outerContextObject();
     void contextObjectHierarchy();
     void destroyContextProperty();
+    void destroyContextObject();
 
     void numericContextProperty();
+    void gcDeletesContextObject();
 
 private:
     QQmlEngine engine;
@@ -1006,6 +983,30 @@ void tst_qqmlcontext::destroyContextProperty()
     // TODO: Or are we?
 }
 
+void tst_qqmlcontext::destroyContextObject()
+{
+    QQmlEngine engine;
+    QList<QQmlRefPointer<QQmlContextData>> contexts;
+    QQmlComponent component(&engine, testFileUrl("destroyContextObject.qml"));
+    QScopedPointer<QObject> root(component.create());
+
+    QPointer<QObject> a = root->property("a").value<QObject *>();
+    QVERIFY(a);
+
+    for (QQmlRefPointer<QQmlContextData> context = QQmlData::get(a)->ownContext;
+         context; context = context->parent()) {
+        contexts.append(context);
+    }
+
+    QObject *deleted = a.data();
+    root.reset();
+
+    QVERIFY(a.isNull());
+
+    for (const auto &context : contexts)
+        QVERIFY(context->contextObject() != deleted);
+}
+
 void tst_qqmlcontext::numericContextProperty()
 {
     QQmlEngine engine;
@@ -1014,6 +1015,33 @@ void tst_qqmlcontext::numericContextProperty()
     QCOMPARE(context->contextProperty(QLatin1String("11")).toInt(), 42);
 }
 
+void tst_qqmlcontext::gcDeletesContextObject()
+{
+    QQmlEngine engine;
+    QQmlComponent c(&engine, testFileUrl("gcDeletesContextObject.qml"));
+
+    QVERIFY2(c.isReady(), qPrintable(c.errorString()));
+    QScopedPointer<QObject> o(c.create());
+
+    QVERIFY(!o.isNull());
+
+    QPointer<QObject> contextObject = o->property("o").value<QObject *>();
+    QVERIFY(contextObject != nullptr);
+
+    QQmlData *data = QQmlData::get(contextObject);
+    QVERIFY(data);
+    QQmlRefPointer<QQmlContextData> context = data->ownContext;
+    QVERIFY(context);
+    QCOMPARE(context->contextObject(), contextObject);
+
+    o->setProperty("o", QVariant::fromValue<QObject *>(nullptr));
+    QCOMPARE(o->property("o").value<QObject *>(), nullptr);
+    engine.collectGarbage();
+
+    QTRY_VERIFY(contextObject.isNull());
+    QCOMPARE(context->contextObject(), nullptr);
+}
+
 QTEST_MAIN(tst_qqmlcontext)
 
 #include "tst_qqmlcontext.moc"
