@@ -1,34 +1,10 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 #include <qtest.h>
 #include <QtQuick/private/qquickitem_p.h>
 #include <QtQuick/private/qquicktext_p.h>
 #include <QtQuick/private/qquickanimation_p.h>
+#include <QtQuick/private/qquicklistview_p.h>
 #include <QtQml/private/qqmlengine_p.h>
 #include <QtQmlModels/private/qqmllistmodel_p.h>
 #include <QtQml/private/qqmlexpression_p.h>
@@ -41,6 +17,8 @@
 
 #include <QtQuickTestUtils/private/qmlutils_p.h>
 
+using namespace Qt::StringLiterals;
+
 Q_DECLARE_METATYPE(QList<int>)
 Q_DECLARE_METATYPE(QList<QVariantHash>)
 
@@ -139,6 +117,7 @@ private slots:
     void listElementWithTemplateString();
     void destroyComponentObject();
     void objectOwnershipFlip();
+    void enumsInListElement();
     void protectQObjectFromGC();
     void deadModelData();
 };
@@ -151,10 +130,10 @@ bool tst_qqmllistmodel::compareVariantList(const QVariantList &testList, QVarian
     if (model == nullptr)
         return false;
 
-    if (model->count() != testList.count())
+    if (model->count() != testList.size())
         return false;
 
-    for (int i=0 ; i < testList.count() ; ++i) {
+    for (int i=0 ; i < testList.size() ; ++i) {
         const QVariant &testVariant = testList.at(i);
         if (testVariant.typeId() != QMetaType::QVariantMap)
             return false;
@@ -620,7 +599,7 @@ void tst_qqmllistmodel::dynamic()
     QCOMPARE(actual,result);
 
     if (model.count() > 0)
-        QVERIFY(spyCount.count() > 0);
+        QVERIFY(spyCount.size() > 0);
 }
 
 void tst_qqmllistmodel::enumerate()
@@ -728,7 +707,7 @@ void tst_qqmllistmodel::error()
     } else {
         QVERIFY(component.isError());
         QList<QQmlError> errors = component.errors();
-        QCOMPARE(errors.count(),1);
+        QCOMPARE(errors.size(),1);
         QCOMPARE(errors.at(0).description(),error);
     }
 }
@@ -743,9 +722,8 @@ void tst_qqmllistmodel::syncError()
     component.setData(qml.toUtf8(),
                       QUrl::fromLocalFile(QString("dummy.qml")));
     QTest::ignoreMessage(QtWarningMsg,error.toUtf8());
-    QObject *obj = component.create();
-    QVERIFY(obj);
-    delete obj;
+    QScopedPointer<QObject> obj(component.create());
+    QVERIFY2(obj, qPrintable(component.errorString()));
 }
 
 /*
@@ -827,7 +805,7 @@ void tst_qqmllistmodel::get()
         QCOMPARE(model->data(index, role), roleValue);
     }
 
-    QCOMPARE(spy.count(), 1);
+    QCOMPARE(spy.size(), 1);
 
     QList<QVariant> spyResult = spy.takeFirst();
     QCOMPARE(spyResult.at(0).value<QModelIndex>(), model->index(index, 0, QModelIndex()));
@@ -928,7 +906,7 @@ void tst_qqmllistmodel::get_nested()
     testData << qMakePair(1, QString("listRoleB"));
     testData << qMakePair(1, QString("listRoleC"));
 
-    for (int i=0; i<testData.count(); i++) {
+    for (int i=0; i<testData.size(); i++) {
         int outerListIndex = testData[i].first;
         QString outerListRoleName = testData[i].second;
         int outerListRole = roleFromName(model, outerListRoleName);
@@ -951,7 +929,7 @@ void tst_qqmllistmodel::get_nested()
         } else {
             QCOMPARE(childModel->data(index, role), roleValue);
         }
-        QCOMPARE(spy.count(), 1);
+        QCOMPARE(spy.size(), 1);
 
         QList<QVariant> spyResult = spy.takeFirst();
         QCOMPARE(spyResult.at(0).value<QModelIndex>(), childModel->index(index, 0, QModelIndex()));
@@ -1095,12 +1073,10 @@ void tst_qqmllistmodel::set_model_cache()
 {
     QQmlEngine eng;
     QQmlComponent component(&eng, testFileUrl("setmodelcachelist.qml"));
-    QObject *model = component.create();
-    QVERIFY2(component.errorString().isEmpty(), QTest::toString(component.errorString()));
+    QScopedPointer<QObject> model(component.create());
+    QVERIFY2(component.errorString().isEmpty(), qPrintable(component.errorString()));
     QVERIFY(model != nullptr);
     QVERIFY(model->property("ok").toBool());
-
-    delete model;
 }
 
 void tst_qqmllistmodel::property_changes()
@@ -1121,9 +1097,9 @@ void tst_qqmllistmodel::property_changes()
 
     QQmlExpression expr(engine.rootContext(), &model, script_setup);
     expr.evaluate();
-    QVERIFY2(!expr.hasError(), QTest::toString(expr.error().toString()));
+    QVERIFY2(!expr.hasError(), qPrintable(expr.error().toString()));
 
-    QString signalHandler = "on" + QString(roleName[0].toUpper()) + roleName.mid(1, roleName.length()) + "Changed:";
+    QString signalHandler = "on" + QString(roleName[0].toUpper()) + roleName.mid(1, roleName.size()) + "Changed:";
     QString qml = "import QtQuick 2.0\n"
                   "Connections {\n"
                         "property bool gotSignal: false\n"
@@ -1135,24 +1111,24 @@ void tst_qqmllistmodel::property_changes()
     component.setData(qml.toUtf8(), QUrl::fromLocalFile(""));
     engine.rootContext()->setContextProperty("model", &model);
     QObject *connectionsObject = component.create();
-    QVERIFY2(component.errorString().isEmpty(), QTest::toString(component.errorString()));
+    QVERIFY2(component.errorString().isEmpty(), qPrintable(component.errorString()));
 
     QSignalSpy spyItemsChanged(&model, SIGNAL(dataChanged(QModelIndex,QModelIndex,QVector<int>)));
 
     expr.setExpression(script_change);
     expr.evaluate();
-    QVERIFY2(!expr.hasError(), QTest::toString(expr.error().toString()));
+    QVERIFY2(!expr.hasError(), qPrintable(expr.error().toString()));
 
     // test the object returned by get() emits the correct signals
     QCOMPARE(connectionsObject->property("gotSignal").toBool(), itemsChanged);
 
     // test itemsChanged() is emitted correctly
     if (itemsChanged) {
-        QCOMPARE(spyItemsChanged.count(), 1);
+        QCOMPARE(spyItemsChanged.size(), 1);
         QCOMPARE(spyItemsChanged.at(0).at(0).value<QModelIndex>(), model.index(listIndex, 0, QModelIndex()));
         QCOMPARE(spyItemsChanged.at(0).at(1).value<QModelIndex>(), model.index(listIndex, 0, QModelIndex()));
     } else {
-        QCOMPARE(spyItemsChanged.count(), 0);
+        QCOMPARE(spyItemsChanged.size(), 0);
     }
 
     expr.setExpression(testExpression);
@@ -1293,15 +1269,13 @@ void tst_qqmllistmodel::signal_handlers()
 
     QQmlEngine eng;
     QQmlComponent component(&eng, testFileUrl("signalhandlers.qml"));
-    QObject *model = component.create();
-    QQmlListModel *lm = qobject_cast<QQmlListModel *>(model);
+    QScopedPointer<QObject> model(component.create());
+    QQmlListModel *lm = qobject_cast<QQmlListModel *>(model.data());
     QVERIFY(lm != nullptr);
     lm->setDynamicRoles(dynamicRoles);
-    QVERIFY2(component.errorString().isEmpty(), QTest::toString(component.errorString()));
+    QVERIFY2(component.errorString().isEmpty(), qPrintable(component.errorString()));
     QVERIFY(model != nullptr);
     QVERIFY(model->property("ok").toBool());
-
-    delete model;
 }
 
 void tst_qqmllistmodel::role_mode_data()
@@ -1576,6 +1550,8 @@ void tst_qqmllistmodel::modify_through_delegate()
         "       ListElement { name: \"Doe\"; age: 33 }\n"
         "   }\n"
         "   ListView {\n"
+        "       height: 100\n" \
+        "       width: 100\n" \
         "       model: testModel\n"
         "       delegate: Item {\n"
         "           Component.onCompleted: model.age = 18;\n"
@@ -1583,7 +1559,7 @@ void tst_qqmllistmodel::modify_through_delegate()
         "   }\n"
         "}\n", QUrl());
 
-    QObject *scene = component.create();
+    QScopedPointer<QObject> scene(component.create());
     QQmlListModel *model = scene->findChild<QQmlListModel*>("testModel");
 
     const QHash<int, QByteArray> roleNames = model->roleNames();
@@ -1621,7 +1597,7 @@ void tst_qqmllistmodel::stringifyModelEntry()
     QQmlListModel *model = scene->findChild<QQmlListModel*>("testModel");
     QQmlExpression expr(engine.rootContext(), model, "JSON.stringify(get(0));");
     QVariant v = expr.evaluate();
-    QVERIFY2(!expr.hasError(), QTest::toString(expr.error().toString()));
+    QVERIFY2(!expr.hasError(), qPrintable(expr.error().toString()));
     const QString expectedString = QStringLiteral("{\"age\":22,\"name\":\"Joe\"}");
     QCOMPARE(v.toString(), expectedString);
 }
@@ -1643,7 +1619,7 @@ void tst_qqmllistmodel::qobjectTrackerForDynamicModelObjects()
     QQmlListModel *model = scene->findChild<QQmlListModel*>("testModel");
     QQmlExpression expr(engine.rootContext(), model, "get(0);");
     QVariant v = expr.evaluate();
-    QVERIFY2(!expr.hasError(), QTest::toString(expr.error().toString()));
+    QVERIFY2(!expr.hasError(), qPrintable(expr.error().toString()));
 
     QObject *obj = v.value<QObject*>();
     QVERIFY(obj);
@@ -1670,8 +1646,8 @@ void tst_qqmllistmodel::crash_append_empty_array()
     QSignalSpy spy(model, &QQmlListModel::rowsAboutToBeInserted);
     QQmlExpression expr(engine.rootContext(), model, "append(new Array())");
     expr.evaluate();
-    QVERIFY2(!expr.hasError(), QTest::toString(expr.error().toString()));
-    QCOMPARE(spy.count(), 0);
+    QVERIFY2(!expr.hasError(), qPrintable(expr.error().toString()));
+    QCOMPARE(spy.size(), 0);
 }
 
 void tst_qqmllistmodel::dynamic_roles_crash_QTBUG_38907()
@@ -1715,7 +1691,7 @@ void tst_qqmllistmodel::nestedListModelIteration()
                 }
             })",
             QUrl());
-    QScopedPointer<QObject>(component.create());
+    delete component.create();
 }
 
 // QTBUG-63569
@@ -1743,7 +1719,7 @@ void tst_qqmllistmodel::undefinedAppendShouldCauseError()
             QUrl());
     QTest::ignoreMessage(QtMsgType::QtWarningMsg, "<Unknown File>: faulty is undefined. Adding an object with a undefined member does not create a role for it.");
     QTest::ignoreMessage(QtMsgType::QtWarningMsg, "<Unknown File>: faulty is null. Adding an object with a null member does not create a role for it.");
-    QScopedPointer<QObject>(component.create());
+    delete component.create();
 }
 
 // QTBUG-89173
@@ -1765,7 +1741,7 @@ void tst_qqmllistmodel::nullPropertyCrash()
             })",
             QUrl());
     QTest::ignoreMessage(QtMsgType::QtWarningMsg, "<Unknown File>: c is null. Adding an object with a null member does not create a role for it.");
-    QScopedPointer<QObject>(component.create());
+    delete component.create();
 }
 
 // QTBUG-91390
@@ -1781,21 +1757,20 @@ void tst_qqmllistmodel::objectDestroyed()
                    })",
             QUrl());
 
-    QObject *obj = new QObject;
-    bool destroyed = false;
-    connect(obj, &QObject::destroyed, [&]() { destroyed = true; });
+    std::unique_ptr<QObject> obj = std::make_unique<QObject>();
+    connect(obj.get(), &QObject::destroyed, [&]() { obj.release(); });
 
-    engine.rootContext()->setContextProperty(u"contextObject"_qs, obj);
-    engine.setObjectOwnership(obj, QJSEngine::JavaScriptOwnership);
+    engine.rootContext()->setContextProperty(u"contextObject"_s, obj.get());
+    engine.setObjectOwnership(obj.get(), QJSEngine::JavaScriptOwnership);
 
-    QScopedPointer<QObject>(component.create());
-    QVERIFY(!destroyed);
+    delete component.create();
+    QVERIFY(obj);
     engine.collectGarbage();
     QTest::qSleep(250);
-    QVERIFY(!destroyed);
-    engine.evaluate(u"model.clear();"_qs);
+    QVERIFY(obj);
+    engine.evaluate(u"model.clear();"_s);
     engine.collectGarbage();
-    QTRY_VERIFY(destroyed);
+    QTRY_VERIFY(!obj);
 }
 
 void tst_qqmllistmodel::destroyObject()
@@ -1811,7 +1786,7 @@ void tst_qqmllistmodel::destroyObject()
                 QUrl());
     QVERIFY2(component.isReady(), qPrintable(component.errorString()));
     QScopedPointer<QObject> element(new QObject);
-    engine.rootContext()->setContextProperty(u"contextObject"_qs, element.data());
+    engine.rootContext()->setContextProperty(u"contextObject"_s, element.data());
 
     QScopedPointer<QObject> o(component.create());
     QVERIFY(!o.isNull());
@@ -1922,6 +1897,22 @@ void tst_qqmllistmodel::objectOwnershipFlip()
     QCOMPARE(QJSEngine::objectOwnership(item.data()), QJSEngine::CppOwnership);
 }
 
+void tst_qqmllistmodel::enumsInListElement()
+{
+    QQmlEngine engine;
+    QQmlComponent component(&engine, testFileUrl("enumsInListElement.qml"));
+    QVERIFY2(component.isReady(), qPrintable(component.errorString()));
+    QScopedPointer<QObject> root(component.create());
+    QVERIFY(!root.isNull());
+
+    QQuickListView *listView = qobject_cast<QQuickListView *>(root.data());
+    QVERIFY(listView);
+    QCOMPARE(listView->count(), 3);
+    for (int i = 0; i < 3; ++i) {
+        QCOMPARE(listView->itemAtIndex(i)->property("text"), QVariant(QString::number(i)));
+    }
+}
+
 void tst_qqmllistmodel::protectQObjectFromGC()
 {
     QQmlEngine engine;
